from pathlib import Path

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{clark-etal-2023-seahorse,
    title = "{SEAHORSE}: A Multilingual, Multifaceted Dataset for Summarization Evaluation",
    author = "Clark, Elizabeth  and
      Rijhwani, Shruti  and
      Gehrmann, Sebastian  and
      Maynez, Joshua  and
      Aharoni, Roee  and
      Nikolaev, Vitaly  and
      Sellam, Thibault  and
      Siddhant, Aditya  and
      Das, Dipanjan  and
      Parikh, Ankur",
    editor = "Bouamor, Houda  and
      Pino, Juan  and
      Bali, Kalika",
    booktitle = "Proceedings of the 2023 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2023",
    address = "Singapore",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.emnlp-main.584",
    doi = "10.18653/v1/2023.emnlp-main.584",
    pages = "9397--9413",
}
"""

_DATASETNAME = "seahorse"

_DESCRIPTION = """
SEAHORSE is a dataset for multilingual, multifaceted summarization evaluation. It consists of 96K summaries with human
ratings along 6 quality dimensions: comprehensibility, repetition, grammar, attribution, main idea(s), and conciseness,
covering 6 languages, 9 systems and 4 datasets.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/seahorse"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = "https://storage.googleapis.com/seahorse-public/seahorse_data.zip"

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


# The original dataset only contaions gem_id, we need to retrieve the article following https://github.com/google-research-datasets/seahorse?tab=readme-ov-file#retrieving-articles-from-gem
def get_wikilingual_data(lang, split):
    ds = datasets.load_dataset("gem", name=f"wiki_lingua_{lang}", split=split)
    df = ds.to_pandas()
    return dict(zip(*[df[col] for col in ["gem_id", "source"]]))


def get_xlsum_data(lang, split):
    df = datasets.load_dataset("GEM/xlsum", lang)
    return {item["gem_id"]: item["text"] for item in df[split]}


# Both train and validation splits in seahorse are taken from the validation split from the original dataset
_WIKILINGUAL_DATA = {split: get_wikilingual_data("vietnamese_vi", split) for split in ["test", "validation"]}
_XLSUM_DATA = {split: get_xlsum_data("vietnamese", split) for split in ["test", "validation"]}


def get_article(gem_id, split):
    if "wiki_lingua" in gem_id:
        data = _WIKILINGUAL_DATA
    elif "xlsum" in gem_id:
        data = _XLSUM_DATA
    else:
        raise AssertionError("gem_id should either from wiki_lingua or xlsum.")
    return data[split if split == "test" else "validation"][gem_id]


class SeahorseDataset(datasets.GeneratorBasedBuilder):
    """Seahorse is a dataset for multilingual, multifaceted summarization evaluation."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "gem_id": datasets.Value("string"),
                    "summary": datasets.Value("string"),
                    "model": datasets.Value("string"),
                    "question1": datasets.Value("string"),
                    "question2": datasets.Value("string"),
                    "question3": datasets.Value("string"),
                    "question4": datasets.Value("string"),
                    "question5": datasets.Value("string"),
                    "question6": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> list[datasets.SplitGenerator]:
        data_dir = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": f"{data_dir}/seahorse_data/train.tsv",
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": f"{data_dir}/seahorse_data/validation.tsv",
                    "split": "dev",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": f"{data_dir}/seahorse_data/test.tsv",
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> tuple[int, dict]:
        df = pd.read_csv(filepath, sep="\t")
        mask = df["worker_lang"] == "vi"
        df_vi = df[mask]
        if self.config.schema == "source":
            for i, row in df_vi.iterrows():
                yield i, {
                    "gem_id": row["gem_id"],
                    "summary": row["summary"],
                    "model": row["model"],
                    "question1": row["question1"],
                    "question2": row["question2"],
                    "question3": row["question3"],
                    "question4": row["question4"],
                    "question5": row["question5"],
                    "question6": row["question6"],
                }

        elif self.config.schema == "seacrowd_t2t":
            for i, row in df_vi.iterrows():
                yield i, {
                    "id": str(i),
                    "text_1": get_article(row["gem_id"], split),
                    "text_2": row["summary"],
                    "text_1_name": "article",
                    "text_2_name": "summary",
                }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = r"""
@inproceedings{chaudhary-etal-2019-low,
    title = "Low-Resource Corpus Filtering Using Multilingual Sentence Embeddings",
    author = "Chaudhary, Vishrav  and
      Tang, Yuqing  and
      Guzm{\'a}n, Francisco  and
      Schwenk, Holger  and
      Koehn, Philipp",
    editor = "Bojar, Ond{\v{r}}ej  and
      Chatterjee, Rajen  and
      Federmann, Christian  and
      Fishel, Mark  and
      Graham, Yvette  and
      Haddow, Barry  and
      Huck, Matthias  and
      Yepes, Antonio Jimeno  and
      Koehn, Philipp  and
      Martins, Andr{\'e}  and
      Monz, Christof  and
      Negri, Matteo  and
      N{\'e}v{\'e}ol, Aur{\'e}lie  and
      Neves, Mariana  and
      Post, Matt  and
      Turchi, Marco  and
      Verspoor, Karin",
    booktitle = "Proceedings of the Fourth Conference on Machine Translation (Volume 3: Shared Task Papers, Day 2)",
    month = aug,
    year = "2019",
    address = "Florence, Italy",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/W19-5435",
    doi = "10.18653/v1/W19-5435",
    pages = "261--266",
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "jav", "sun", "tha", "vie", "zlm", "lao", "khm", "mya", "ceb"]
_DATASETNAME = "cc_aligned_sent"
_DESCRIPTION = """\
This dataset contains the sentence pairs extracted from CC-Aligned document
pairs using similarity scores of LASER embeddings (minimum similarity 1.04,
sorted based on decreasing similarity score). It misses some languages not
covered by LASER.
"""

_HOMEPAGE = "https://www2.statmt.org/cc-aligned/"
_LICENSE = Licenses.UNKNOWN.value
_URL = "https://data.statmt.org/cc-aligned/sentence-aligned/"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_SUBSETS = ["id_ID", "jv_ID", "su_ID", "th_TH", "vi_VN", "ms_MY", "lo_LA", "km_KH", "my_MM", "cx_PH"]


class CCAlignedSentencesDataset(datasets.GeneratorBasedBuilder):
    """CC Aligned Sentences dataset by Chaudhary et al., (2019)"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    # Add configurations for loading a dataset per language.
    dataset_names = sorted([f"{_DATASETNAME}_{subset}" for subset in _SUBSETS])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        source_config = SEACrowdConfig(
            name=f"{name}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(source_config)
        seacrowd_config = SEACrowdConfig(
            name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(seacrowd_config)

    # Choose first language as default
    first_subset = sorted(_SUBSETS)[0]
    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_{first_subset}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "Source_Sentence": datasets.Value("string"),
                    "Target_Sentence": datasets.Value("string"),
                    "LASER_similarity": datasets.Value("float64"),
                }
            )

        if self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_to_text.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Return SplitGenerators."""
        # Define some functions for parsing config and URL names
        def _split_at_n(text: str, n: int) -> Tuple[str, str]:
            """Split text on the n-th instance"""
            return ("_".join(text.split("_")[:n]), "_".join(text.split("_")[n:]))

        # Get URL. For cx_PH, the source and target languages are reversed
        _, subset = _split_at_n(_split_at_n(self.config.name, 5)[0], 3)
        (source_lang, target_lang) = (subset, "en_XX") if subset == "cx_PH" else ("en_XX", subset)
        url = _URL + f"{source_lang}-{target_lang}.tsv.xz"
        filepath = dl_manager.download_and_extract(url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": filepath,
                    "source_lang": source_lang,
                    "target_lang": target_lang,
                },
            )
        ]

    def _generate_examples(self, filepath: Path, source_lang: str, target_lang: str) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        with open(filepath, encoding="utf-8") as file:
            for idx, row in enumerate(file):
                text_1, text_2, score = row.strip().split("\t")
                if self.config.schema == "source":
                    example = {
                        "id": idx,
                        "Source_Sentence": text_1,
                        "Target_Sentence": text_2,
                        "LASER_similarity": float(score),
                    }
                if self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    example = {
                        "id": idx,
                        "text_1": text_1,
                        "text_2": text_2,
                        "text_1_name": source_lang,
                        "text_2_name": target_lang,
                    }
                yield idx, example



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{cruz2021exploiting,
  title={Exploiting news article structure for automatic corpus generation of entailment datasets},
  author={Cruz, Jan Christian Blaise and Resabal, Jose Kristian and Lin, James and Velasco, Dan John and Cheng, Charibeth},
  booktitle={PRICAI 2021: Trends in Artificial Intelligence: 18th Pacific Rim International Conference on Artificial Intelligence, PRICAI 2021, Hanoi, Vietnam, November 8--12, 2021, Proceedings, Part II 18},
  pages={86--99},
  year={2021},
  organization={Springer}
}
"""
_DATASETNAME = "newsph"
_LANGUAGES = ["fil", "tgl"]
_DESCRIPTION = """\
Raw collection of news articles in Filipino which can be used for language modelling.
"""
_HOMEPAGE = "https://huggingface.co/datasets/newsph"
_LICENSE = Licenses.GPL_3_0.value
_LOCAL = False
_URLS = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/newsph/newsph.zip"
_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class NewsPhDataset(datasets.GeneratorBasedBuilder):
    """
    Raw collection of news articles in Filipino which can be used for language modelling.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="newsph_source",
            version=SOURCE_VERSION,
            description="newsph source schema",
            schema="source",
            subset_id="newsph",
        ),
        SEACrowdConfig(
            name="newsph_seacrowd_ssp",
            version=SEACROWD_VERSION,
            description="newsph SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id="newsph",
        ),
    ]

    DEFAULT_CONFIG_NAME = "newsph_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features
        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_dir = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "newsph", "train.txt"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema == "source" or self.config.schema == "seacrowd_ssp":
            with open(filepath, encoding="utf-8") as f:
                for idx, row in enumerate(f):
                    if row.strip():
                        yield idx, {"id": str(idx), "text": row}
                    else:
                        yield idx, {"id": str(idx), "text": ""}
        else:
            raise NotImplementedError



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """
"""

_DATASETNAME = "id_sts"

_DESCRIPTION = """\
SemEval is a series of international natural language processing (NLP) research workshops whose mission is
to advance the current state of the art in semantic analysis and to help create high-quality annotated datasets in a
range of increasingly challenging problems in natural language semantics. This is a translated version of SemEval Dataset
from 2012-2016 for Semantic Textual Similarity Task to Indonesian language.
"""

_HOMEPAGE = "https://github.com/ahmadizzan/sts-indo"

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/ahmadizzan/sts-indo/master/data/final-data/train.tsv",
        "test": "https://raw.githubusercontent.com/ahmadizzan/sts-indo/master/data/final-data/test.tsv",
    }
}

_SUPPORTED_TASKS = [Tasks.SEMANTIC_SIMILARITY]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IdSts(datasets.GeneratorBasedBuilder):
    """id_sts, translated version of SemEval Dataset
    from 2012-2016 for Semantic Textual Similarity Task to Indonesian language"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_sts_source",
            version=SOURCE_VERSION,
            description="ID_STS source schema",
            schema="source",
            subset_id="id_sts",
        ),
        SEACrowdConfig(
            name="id_sts_seacrowd_pairs_score",
            version=SEACROWD_VERSION,
            description="ID_STS Nusantara schema",
            schema="seacrowd_pairs_score",
            subset_id="id_sts",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_sts_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text_1": datasets.Value("string"),
                    "text_2": datasets.Value("string"),
                    "label": datasets.Value("float64"),
                }
            )
        elif self.config.schema == "seacrowd_pairs_score":
            features = schemas.pairs_features_score()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        train_data_path = Path(dl_manager.download(urls["train"]))
        test_data_path = Path(dl_manager.download(urls["test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_data_path, "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_data_path, "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        df = pd.read_csv(filepath, delimiter="\t").reset_index()
        df.columns = ["id", "score", "original_text_1", "original_text_2", "source", "text_1", "text_2"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"text_1": row.text_1, "text_2": row.text_2, "label": row.score}
                yield row.id, ex

        elif self.config.schema == "seacrowd_pairs_score":
            for row in df.itertuples():
                ex = {"id": str(row.id), "text_1": row.text_1, "text_2": row.text_2, "label": row.score}
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{imperial2019sentiment,
      title={Sentiment Analysis of Typhoon Related Tweets using Standard and Bidirectional Recurrent Neural Networks}, 
      author={Joseph Marvin Imperial and Jeyrome Orosco and Shiela Mae Mazo and Lany Maceda},
      year={2019},
      eprint={1908.01765},
      archivePrefix={arXiv},
      primaryClass={cs.NE}
}
"""

_DATASETNAME = "typhoon_yolanda_tweets"

_DESCRIPTION = """\
The dataset contains annotated typhoon and disaster-related tweets in Filipino collected before, during, 
and after one month of Typhoon Yolanda in 2013. The dataset has been annotated by an expert into three 
sentiment categories: positive, negative, and neutral.
"""

_HOMEPAGE = "https://github.com/imperialite/Philippine-Languages-Online-Corpora/tree/master/Tweets/Annotated%20Yolanda"

_LOCAL = False
_LANGUAGES = ["fil"]

_LICENSE = Licenses.CC_BY_4_0.value

_ROOT_URL = "https://raw.githubusercontent.com/imperialite/Philippine-Languages-Online-Corpora/master/Tweets/Annotated%20Yolanda/"
_URLS = {"train": {-1: _ROOT_URL + "train/-1.txt", 0: _ROOT_URL + "train/0.txt", 1: _ROOT_URL + "train/1.txt"}, "test": {-1: _ROOT_URL + "test/-1.txt", 0: _ROOT_URL + "test/0.txt", 1: _ROOT_URL + "test/1.txt"}}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class TyphoonYolandaTweets(datasets.GeneratorBasedBuilder):
    """
    The dataset contains annotated typhoon and disaster-related tweets in Filipino collected before, during, and
    after one month of Typhoon Yolanda in 2013. The dataset has been annotated by an expert into three sentiment
    categories: positive, negative, and neutral.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="typhoon_yolanda_tweets_source",
            version=SOURCE_VERSION,
            description="Typhoon Yolanda Tweets source schema",
            schema="source",
            subset_id="typhoon_yolanda_tweets",
        ),
        SEACrowdConfig(
            name="typhoon_yolanda_tweets_seacrowd_text",
            version=SEACROWD_VERSION,
            description="Typhoon Yolanda Tweets SEACrowd schema",
            schema="seacrowd_text",
            subset_id="typhoon_yolanda_tweets",
        ),
    ]

    DEFAULT_CONFIG_NAME = "typhoon_yolanda_tweets_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["-1", "0", "1"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        emos = [-1, 0, 1]
        if self.config.name == "typhoon_yolanda_tweets_source" or self.config.name == "typhoon_yolanda_tweets_seacrowd_text":
            train_path = dl_manager.download_and_extract({emo: _URLS["train"][emo] for emo in emos})

            test_path = dl_manager.download_and_extract({emo: _URLS["test"][emo] for emo in emos})

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_path,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema != "source" and self.config.schema != "seacrowd_text":
            raise ValueError(f"Invalid config: {self.config.name}")

        df = pd.DataFrame(columns=["text", "label"])

        if self.config.name == "typhoon_yolanda_tweets_source" or self.config.name == "typhoon_yolanda_tweets_seacrowd_text":
            for emo, file in filepath.items():
                with open(file) as f:
                    t = f.readlines()
                    l = [str(emo)]*(len(t))
                    tmp_df = pd.DataFrame.from_dict({"text": t, "label": l})
                    df = pd.concat([df, tmp_df], ignore_index=True)

        for row in df.itertuples():
            ex = {"id": str(row.Index), "text": row.text, "label": row.label}
            yield row.Index, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Astuti2023,
title = {Code-Mixed Sentiment Analysis using Transformer for Twitter Social Media Data},
journal = {International Journal of Advanced Computer Science and Applications},
doi = {10.14569/IJACSA.2023.0141053},
url = {http://dx.doi.org/10.14569/IJACSA.2023.0141053},
year = {2023},
publisher = {The Science and Information Organization},
volume = {14},
number = {10},
author = {Laksmita Widya Astuti and Yunita Sari and Suprapto}
}
"""

_DATASETNAME = "indonglish"
_DESCRIPTION = """\
Indonglish-dataset was constructed based on keywords derived from the
sociolinguistic phenomenon observed among teenagers in South Jakarta. The
dataset was designed to tackle the semantic task of sentiment analysis,
incorporating three distinct label categories: positive, negative, and
neutral. The annotation of the dataset was carried out by a panel of five
annotators, each possessing expertise language and data science.
"""

_HOMEPAGE = "https://github.com/laksmitawidya/indonglish-dataset"
_LANGUAGES = ["ind"]
_LICENSE = Licenses.UNKNOWN.value
_LOCAL = False

_URLS = {
    "skenario-orig": {
        "train": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario-ori/train.csv",
        "validation": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario-ori/validation.csv",
        "test": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario-ori/test.csv",
    },
    "skenario1": {
        "train": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario1/training.csv",
        "validation": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario1/validation.csv",
        "test": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario1/test.csv",
    },
    "skenario2": {
        "train": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario2/training.csv",
        "validation": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario2/validation.csv",
        "test": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario2/test.csv",
    },
    "skenario3": {
        "train": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario3/training.csv",
        "validation": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario3/validation.csv",
        "test": "https://raw.githubusercontent.com/laksmitawidya/indonglish-dataset/master/skenario3/test.csv",
    },
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class Indonglish(datasets.GeneratorBasedBuilder):
    """Indonglish dataset for sentiment analysis from https://github.com/laksmitawidya/indonglish-dataset."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"
    _LABELS = ["Positif", "Negatif", "Netral"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]
    for i in range(1, 4):
        BUILDER_CONFIGS += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_skenario{i}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema",
                schema="source",
                subset_id=f"{_DATASETNAME}_skenario{i}",
            ),
            SEACrowdConfig(
                name=f"{_DATASETNAME}_skenario{i}_seacrowd_{SEACROWD_SCHEMA_NAME}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} SEACrowd schema",
                schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
                subset_id=f"{_DATASETNAME}_skenario{i}",
            ),
        ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tweet": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=self._LABELS),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(self._LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        if "skenario" in self.config.name:
            setting = self.config.name.split("_")[1]
        else:
            setting = "skenario-orig"

        data_paths = {
            setting: {
                "train": Path(dl_manager.download_and_extract(_URLS[setting]["train"])),
                "validation": Path(dl_manager.download_and_extract(_URLS[setting]["validation"])),
                "test": Path(dl_manager.download_and_extract(_URLS[setting]["test"])),
            }
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_paths[setting]["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_paths[setting]["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_paths[setting]["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        # read csv file
        with open(filepath, "r", encoding="utf-8") as csv_file:
            csv_reader = csv.reader(csv_file)
            csv_data = [row for row in csv_reader]
        csv_data = csv_data[1:]  # remove header

        num_sample = len(csv_data)

        for i in range(num_sample):
            if self.config.schema == "source":
                example = {
                    "id": str(i),
                    "tweet": csv_data[i][0],
                    "label": csv_data[i][1],
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                example = {
                    "id": str(i),
                    "text": csv_data[i][0],
                    "label": csv_data[i][1],
                }

            yield i, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_DATASETNAME = "nusax_mt"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind", "ace", "ban", "bjn", "bbc", "bug", "jav", "mad", "min", "nij", "sun", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_CITATION = """\
@misc{winata2022nusax,
      title={NusaX: Multilingual Parallel Sentiment Dataset for 10 Indonesian Local Languages},
      author={Winata, Genta Indra and Aji, Alham Fikri and Cahyawijaya,
      Samuel and Mahendra, Rahmad and Koto, Fajri and Romadhony,
      Ade and Kurniawan, Kemal and Moeljadi, David and Prasojo,
      Radityo Eko and Fung, Pascale and Baldwin, Timothy and Lau,
      Jey Han and Sennrich, Rico and Ruder, Sebastian},
      year={2022},
      eprint={2205.15960},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
NusaX is a high-quality multilingual parallel corpus that covers 12 languages, Indonesian, English, and 10 Indonesian local languages, namely Acehnese, Balinese, Banjarese, Buginese, Madurese, Minangkabau, Javanese, Ngaju, Sundanese, and Toba Batak.

NusaX-MT is a parallel corpus for training and benchmarking machine translation models across 10 Indonesian local languages + Indonesian and English. The data is presented in csv format with 12 columns, one column for each language.
"""

_HOMEPAGE = "https://github.com/IndoNLP/nusax/tree/main/datasets/mt"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_URLS = {
    "train": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/mt/train.csv",
    "validation": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/mt/valid.csv",
    "test": "https://raw.githubusercontent.com/IndoNLP/nusax/main/datasets/mt/test.csv",
}


def seacrowd_config_constructor(lang_source, lang_target, schema, version):
    """Construct SEACrowdConfig with nusax_mt_{lang_source}_{lang_target}_{schema} as the name format"""
    if schema != "source" and schema != "seacrowd_t2t":
        raise ValueError(f"Invalid schema: {schema}")

    if lang_source == "" and lang_target == "":
        return SEACrowdConfig(
            name="nusax_mt_{schema}".format(schema=schema),
            version=datasets.Version(version),
            description="nusax_mt with {schema} schema for all 132 language pairs".format(schema=schema),
            schema=schema,
            subset_id="nusax_mt",
        )
    else:
        return SEACrowdConfig(
            name="nusax_mt_{lang_source}_{lang_target}_{schema}".format(lang_source=lang_source, lang_target=lang_target, schema=schema),
            version=datasets.Version(version),
            description="nusax_mt with {schema} schema for {lang_source} source language and  {lang_target} target language".format(lang_source=lang_source, lang_target=lang_target, schema=schema),
            schema=schema,
            subset_id="nusax_mt",
        )


LANGUAGES_MAP = {
    "ace": "acehnese",
    "ban": "balinese",
    "bjn": "banjarese",
    "bug": "buginese",
    "eng": "english",
    "ind": "indonesian",
    "jav": "javanese",
    "mad": "madurese",
    "min": "minangkabau",
    "nij": "ngaju",
    "sun": "sundanese",
    "bbc": "toba_batak",
}


class NusaXMT(datasets.GeneratorBasedBuilder):
    """NusaX-MT is a parallel corpus for training and benchmarking machine translation models across 10 Indonesian local languages + Indonesian and English. The data is presented in csv format with 12 columns, one column for each language."""

    BUILDER_CONFIGS = (
        [seacrowd_config_constructor(lang1, lang2, "source", _SOURCE_VERSION) for lang1 in LANGUAGES_MAP for lang2 in LANGUAGES_MAP if lang1 != lang2]
        + [seacrowd_config_constructor(lang1, lang2, "seacrowd_t2t", _SEACROWD_VERSION) for lang1 in LANGUAGES_MAP for lang2 in LANGUAGES_MAP if lang1 != lang2]
        + [seacrowd_config_constructor("", "", "source", _SOURCE_VERSION), seacrowd_config_constructor("", "", "seacrowd_t2t", _SEACROWD_VERSION)]
    )

    DEFAULT_CONFIG_NAME = "nusax_senti_ind_eng_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source" or self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features
        else:
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        train_csv_path = Path(dl_manager.download_and_extract(_URLS["train"]))
        validation_csv_path = Path(dl_manager.download_and_extract(_URLS["validation"]))
        test_csv_path = Path(dl_manager.download_and_extract(_URLS["test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": validation_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_csv_path},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema != "source" and self.config.schema != "seacrowd_t2t":
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        df = pd.read_csv(filepath).reset_index()
        if self.config.name == "nusax_mt_source" or self.config.name == "nusax_mt_seacrowd_t2t":
            # load all 132 language pairs
            id_count = -1
            for lang_source in LANGUAGES_MAP:
                for lang_target in LANGUAGES_MAP:
                    if lang_source == lang_target:
                        continue

                    for _, row in df.iterrows():
                        id_count += 1
                        ex = {
                            "id": str(id_count),
                            "text_1": row[LANGUAGES_MAP[lang_source]],
                            "text_2": row[LANGUAGES_MAP[lang_target]],
                            "text_1_name": lang_source,
                            "text_2_name": lang_target,
                        }
                        yield id_count, ex

        else:
            df = pd.read_csv(filepath).reset_index()
            lang_source = self.config.name[9:12]
            lang_target = self.config.name[13:16]

            for index, row in df.iterrows():
                ex = {
                    "id": str(index),
                    "text_1": row[LANGUAGES_MAP[lang_source]],
                    "text_2": row[LANGUAGES_MAP[lang_target]],
                    "text_1_name": lang_source,
                    "text_2_name": lang_target,
                }
                yield str(index), ex



import os
from pathlib import Path
from typing import Dict, List, Tuple
from zipfile import ZipFile

import datasets
import yaml

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{PhoST,
    title     = {{A High-Quality and Large-Scale Dataset for English-Vietnamese Speech Translation}},
    author    = {Linh The Nguyen and Nguyen Luong Tran and Long Doan and Manh Luong and Dat Quoc Nguyen},
    booktitle = {Proceedings of the 23rd Annual Conference of the International Speech Communication Association (INTERSPEECH)},
    year      = {2022}
}
"""

_DATASETNAME = "phost"

_DESCRIPTION = """\
PhoST is a high-quality and large-scale benchmark dataset for English-Vietnamese speech translation
with 508 audio hours, consisting of 331K triplets of (sentence-lengthed audio, English source
transcript sentence, Vietnamese target subtitle sentence).
"""

_HOMEPAGE = "https://github.com/VinAIResearch/PhoST"

_LICENSE = Licenses.CC_BY_NC_ND_4_0.value

_LOCAL = True

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION, Tasks.SPEECH_TO_TEXT_TRANSLATION, Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_LANGUAGES = ["eng", "vie"]


def seacrowd_config_constructor(src_lang, tgt_lang, schema, version):
    if src_lang == "" or tgt_lang == "":
        raise ValueError(f"Invalid src_lang {src_lang} or tgt_lang {tgt_lang}")

    if schema not in ["source", "seacrowd_sptext", "seacrowd_t2t"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="phost_{src}_{tgt}_{schema}".format(src=src_lang, tgt=tgt_lang, schema=schema),
        version=datasets.Version(version),
        description="phost schema for {schema} from {src} to {tgt}".format(schema=schema, src=src_lang, tgt=tgt_lang),
        schema=schema,
        subset_id="phost_{src}_{tgt}".format(src=src_lang, tgt=tgt_lang),
    )


class Phost(datasets.GeneratorBasedBuilder):
    """
    PhoST is a high-quality and large-scale benchmark dataset for English-Vietnamese speech translation
    with 508 audio hours, consisting of 331K triplets of (sentence-lengthed audio, English source
    transcript sentence, Vietnamese target subtitle sentence).
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        seacrowd_config_constructor("en", "vi", "source", _SOURCE_VERSION),
        seacrowd_config_constructor("en", "vi", "seacrowd_sptext", _SEACROWD_VERSION),
        seacrowd_config_constructor("en", "vi", "seacrowd_t2t", _SEACROWD_VERSION),
    ]

    DEFAULT_CONFIG_NAME = "phost_en_vi_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "file": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "en_text": datasets.Value("string"),
                    "vi_text": datasets.Value("string"),
                    "timing": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

            aud_path = os.path.join(data_dir, "audio_data")
            if not os.path.exists(aud_path):
                os.makedirs(aud_path)

            # loading the temp.zip and creating a zip object
            with ZipFile(os.path.join(data_dir, "train_audio.zip"), "r") as zObject:
                for member in zObject.namelist():
                    if not os.path.exists(os.path.join(aud_path, "train", member)) or not os.path.isfile(os.path.join(aud_path, "train", member)):
                        zObject.extract(member, os.path.join(aud_path, "train"))

            # dev audio files
            with ZipFile(os.path.join(data_dir, "dev_audio.zip"), "r") as zObject:
                for member in zObject.namelist():
                    if not os.path.exists(os.path.join(aud_path, "dev", member)) or not os.path.isfile(os.path.join(aud_path, "dev", member)):
                        zObject.extract(member, aud_path)
            # test audio files
            with ZipFile(os.path.join(data_dir, "test_audio.zip"), "r") as zObject:
                for member in zObject.namelist():
                    if not os.path.exists(os.path.join(aud_path, "test", member)) or not os.path.isfile(os.path.join(aud_path, "test", member)):
                        zObject.extract(member, aud_path)
            # text data
            with ZipFile(os.path.join(data_dir, "text_data.zip"), "r") as zObject:
                for member in zObject.namelist():
                    if not os.path.exists(os.path.join(data_dir, member)) or not os.path.isfile(os.path.join(data_dir, member)):
                        zObject.extract(member, data_dir)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {"audio": os.path.join(aud_path, "train", "wav"), "text": os.path.join(data_dir, "text_data", "train")},
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"audio": os.path.join(aud_path, "test", "wav"), "text": os.path.join(data_dir, "text_data", "test")},
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": {"audio": os.path.join(aud_path, "dev", "wav"), "text": os.path.join(data_dir, "text_data", "dev")},
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        config_names_split = self.config.name.split("_")
        src_lang = config_names_split[1]
        tgt_lang = config_names_split[2]
        track_ids = os.listdir(filepath["text"])
        timing = []
        en_sub = []
        vi_sub = []
        counter = 0
        for key, track_id in enumerate(track_ids):
            with open(os.path.join(filepath["text"], track_id, track_id + ".yaml")) as timing_file:
                timing = yaml.safe_load(timing_file)
            with open(os.path.join(filepath["text"], track_id, track_id + ".en")) as en_text:
                en_sub = [line.strip() for line in en_text]
            with open(
                os.path.join(filepath["text"], track_id, track_id + ".vi"),
            ) as vi_text:
                vi_sub = [line.strip() for line in vi_text]

            if self.config.schema == "source":
                yield key, {"file": os.path.join(filepath["audio"], track_id + ".wav"), "audio": os.path.join(filepath["audio"], track_id + ".wav"), "en_text": " ".join(en_sub), "vi_text": " ".join(vi_sub), "timing": timing}

            elif self.config.schema == "seacrowd_sptext":
                if tgt_lang not in ["en", "vi"]:
                    raise NotImplementedError(f"Target language '{tgt_lang}' is not defined.")

                yield key, {
                    "id": track_id,
                    "path": os.path.join(filepath["audio"], track_id + ".wav"),
                    "audio": os.path.join(filepath["audio"], track_id + ".wav"),
                    "text": " ".join(en_sub) if tgt_lang == "en" else " ".join(vi_sub),
                    "speaker_id": None,
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": None,
                    },
                }

            elif self.config.schema == "seacrowd_t2t":
                if src_lang not in ["en", "vi"]:
                    raise NotImplementedError(f"Source language '{src_lang}' is not defined.")
                if tgt_lang not in ["en", "vi"]:
                    raise NotImplementedError(f"Target language '{tgt_lang}' is not defined.")
                for en_line, vi_line in zip(en_sub, vi_sub):
                    yield counter, {
                        "id": f"{track_id}_{str(counter)}",
                        "text_1": en_line if src_lang == "en" else vi_line,
                        "text_2": en_line if tgt_lang == "en" else vi_line,
                        "text_1_name": src_lang,
                        "text_2_name": tgt_lang,
                    }
                    counter += 1
            else:
                raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



import os
from itertools import chain
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{widiaputri-etal-5641,
  author = {Widiaputri, Ruhiyah Faradishi and Purwarianti, Ayu and Lestari, Dessi Puji and Azizah, Kurniawati and Tanaya, Dipta and Sakti, Sakriani},
  title = {Speech Recognition and Meaning Interpretation: Towards Disambiguation of Structurally Ambiguous Spoken Utterances in Indonesian},
  booktitle = {Proceedings of the EMNLP 2023},
  year = {2023}
}
"""

_DATASETNAME = "struct_amb_ind"

_DESCRIPTION = """
This dataset contains the first Indonesian speech dataset for structurally ambiguous utterances and each of transcription and two disambiguation texts.
The structurally ambiguous sentences were adapted from Types 4,5,6, and 10 of Types Of Syntactic Ambiguity in English by [Taha et al., 1983].
For each chosen type, 100 structurally ambiguous sentences in Indonesian were made by crowdsourcing.
Each Indonesian ambiguous sentence has two possible interpretations, resulting in two disambiguation text outputs for each ambiguous sentence.
Each disambiguation text is made up of two sentences. All of the sentences have been checked by linguists.
"""

_HOMEPAGE = "https://github.com/ha3ci-lab/struct_amb_ind"

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = True  # get the audio data externally from https://drive.google.com/drive/folders/1QeaptstBgwGYO6THGkZHHViExrogCMUj
_LANGUAGES = ["ind"]

_URL_TEMPLATES = {
    "keys": "https://raw.githubusercontent.com/ha3ci-lab/struct_amb_ind/main/keys/train_dev_test_spk_keys/",
    "text": "https://raw.githubusercontent.com/ha3ci-lab/struct_amb_ind/main/text/",
}

_URLS = {
    "split_train": _URL_TEMPLATES["keys"] + "train_spk",
    "split_dev": _URL_TEMPLATES["keys"] + "dev_spk",
    "split_test": _URL_TEMPLATES["keys"] + "test_spk",
    "text_transcript": _URL_TEMPLATES["text"] + "ID_amb_disam_transcript.txt",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class StructAmbInd(datasets.GeneratorBasedBuilder):
    """
    This dataset contains the first Indonesian speech dataset for structurally ambiguous utterances and each of transcription and two disambiguation texts.
    This dataloader does NOT contain the additional training data for as mentioned in the _HOMEPAGE, as it is already implemented in the dataloader "indspeech_news_lvcsr".
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_sptext",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "amb_transcript": datasets.Value("string"),
                    "disam_text": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # The data_dir configuration is required ONLY for the audio_urls.
        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        # load the local audio folders
        audio_urls = [data_dir + "/" + f"{gender}{_id:02}.zip" for gender in ["F", "M"] for _id in range(1, 12, 1)]
        audio_files_dir = [Path(dl_manager.extract(audio_url)) / audio_url.split("/")[-1][:-4] for audio_url in audio_urls]
        # load the speaker splits and transcript
        split_train = Path(dl_manager.download(_URLS["split_train"]))
        split_dev = Path(dl_manager.download(_URLS["split_dev"]))
        split_test = Path(dl_manager.download(_URLS["split_test"]))
        text_transcript = Path(dl_manager.download(_URLS["text_transcript"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"split": split_train, "transcript": text_transcript, "audio_files_dir": audio_files_dir},
            ),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"split": split_dev, "transcript": text_transcript, "audio_files_dir": audio_files_dir}),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"split": split_test, "transcript": text_transcript, "audio_files_dir": audio_files_dir},
            ),
        ]

    def _generate_examples(self, split: Path, transcript: Path, audio_files_dir: List[Path]) -> Tuple[int, Dict]:
        speaker_ids = open(split, "r").readlines()
        speaker_ids = [id.replace("\n", "") for id in speaker_ids]
        speech_folders = [audio_folder for audio_folder in audio_files_dir if audio_folder.name.split("/")[-1] in speaker_ids]
        speech_files = list(chain(*[list(map((str(speech_folder) + "/").__add__, os.listdir(speech_folder))) for speech_folder in speech_folders]))

        transcript = open(transcript, "r").readlines()
        transcript = [sent.replace("\n", "").split("|") for sent in transcript]
        transcript_dict = {sent[0]: {"amb_transcript": sent[1], "disam_text": sent[2]} for sent in transcript}

        for key, aud_file in enumerate(speech_files):
            aud_id = aud_file.split("/")[-1][:-4]
            aud_info = aud_id.split("_")
            if self.config.schema == "source":
                row = {
                    "id": aud_id,
                    "speaker_id": aud_info[1],
                    "path": aud_file,
                    "audio": aud_file,
                    "amb_transcript": transcript_dict[aud_id]["amb_transcript"],
                    "disam_text": transcript_dict[aud_id]["disam_text"],
                }
                yield key, row
            elif self.config.schema == "seacrowd_sptext":
                row = {
                    "id": aud_id,
                    "path": aud_file,
                    "audio": aud_file,
                    "text": transcript_dict[aud_id]["amb_transcript"],
                    "speaker_id": aud_info[1],
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": aud_info[1][0],
                    },
                }
                yield key, row
            else:
                raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This template serves as a starting point for contributing a dataset to the Nusantara Dataset repo.

When modifying it for your dataset, look for TODO items that offer specific instructions.

Full documentation on writing dataset loading scripts can be found here:
https://huggingface.co/docs/datasets/add_dataset.html

To create a dataset loading script you will create a class and implement 3 methods:
  * `_info`: Establishes the schema for the dataset, and returns a datasets.DatasetInfo object.
  * `_split_generators`: Downloads and extracts data for each split (e.g. train/val/test) or associate local data with each split.
  * `_generate_examples`: Creates examples from data on disk that conform to each schema defined in `_info`.

TODO: Before submitting your script, delete this doc string and replace it with a description of your dataset.

[seacrowd_schema_name] = (kb, pairs, qa, text, t2t, entailment)
"""
from base64 import encode
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

# TODO: Add BibTeX citation
_CITATION = """\
@article{DBLP:journals/corr/abs-2011-00677,
  author    = {Fajri Koto and
               Afshin Rahimi and
               Jey Han Lau and
               Timothy Baldwin},
  title     = {IndoLEM and IndoBERT: {A} Benchmark Dataset and Pre-trained Language
               Model for Indonesian {NLP}},
  journal   = {CoRR},
  volume    = {abs/2011.00677},
  year      = {2020},
  url       = {https://arxiv.org/abs/2011.00677},
  eprinttype = {arXiv},
  eprint    = {2011.00677},
  timestamp = {Fri, 06 Nov 2020 15:32:47 +0100},
  biburl    = {https://dblp.org/rec/journals/corr/abs-2011-00677.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

# TODO: create a module level variable with your dataset name (should match script name)
#  E.g. Hallmarks of Cancer: [dataset_name] --> hallmarks_of_cancer
_DATASETNAME = "indolem_sentiment"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
IndoLEM (Indonesian Language Evaluation Montage) is a comprehensive Indonesian benchmark that comprises of seven tasks for the Indonesian language. This benchmark is categorized into three pillars of NLP tasks: morpho-syntax, semantics, and discourse.

This dataset is based on binary classification (positive and negative), with distribution:
* Train: 3638 sentences
* Development: 399 sentences
* Test: 1011 sentences

The data is sourced from 1) Twitter [(Koto and Rahmaningtyas, 2017)](https://www.researchgate.net/publication/321757985_InSet_Lexicon_Evaluation_of_a_Word_List_for_Indonesian_Sentiment_Analysis_in_Microblogs)
and 2) [hotel reviews](https://github.com/annisanurulazhar/absa-playground/).

The experiment is based on 5-fold cross validation.
"""

# TODO: Add a link to an official homepage for the dataset here (if possible)
_HOMEPAGE = "https://indolem.github.io/"

# TODO: Add the licence for the dataset here (if possible)
# Note that this doesn't have to be a common open source license.
# Some datasets have custom licenses. In this case, simply put the full license terms
# into `_LICENSE`
_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

# TODO: Add links to the urls needed to download your dataset files.
#  For local datasets, this variable can be an empty dictionary.

# For publicly available datasets you will most likely end up passing these URLs to dl_manager in _split_generators.
# In most cases the URLs will be the same for the source and seacrowd config.
# However, if you need to access different files for each config you can have multiple entries in this dict.
# This can be an arbitrarily nested dict/list of URLs (see below in `_split_generators` method)
_URLS = {
    _DATASETNAME: {
        'train': 'https://raw.githubusercontent.com/indolem/indolem/main/sentiment/data/train0.csv',
        'dev': 'https://raw.githubusercontent.com/indolem/indolem/main/sentiment/data/dev0.csv',
        'test': 'https://raw.githubusercontent.com/indolem/indolem/main/sentiment/data/test0.csv'
    }
}

# TODO: add supported task by dataset. One dataset may support multiple tasks
_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

# TODO: set this to a version that is associated with the dataset. if none exists use "1.0.0"
#  This version doesn't have to be consistent with semantic versioning. Anything that is
#  provided by the original dataset as a version goes.
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


# TODO: Name the dataset class to match the script name using CamelCase instead of snake_case
class IndolemSentimentDataset(datasets.GeneratorBasedBuilder):

    label_classes = ['negative','positive']
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # You will be able to load the "source" or "se" configurations with
    # ds_source = datasets.load_dataset('my_dataset', name='source')
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd')

    # For local datasets you can make use of the `data_dir` and `data_files` kwargs
    # https://huggingface.co/docs/datasets/add_dataset.html#downloading-data-files-and-organizing-splits
    # ds_source = datasets.load_dataset('my_dataset', name='source', data_dir="/path/to/data/files")
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd', data_dir="/path/to/data/files")

    # TODO: For each dataset, implement Config for Source and Nusantara;
    #  If dataset contains more than one subset (see seacrowd/sea_datasets/smsa.py) implement for EACH of them.
    #  Each of them should contain:
    #   - name: should be unique for each dataset config eg. smsa_(source|seacrowd)_[seacrowd_schema_name]
    #   - version: option = (SOURCE_VERSION|SEACROWD_VERSION)
    #   - description: one line description for the dataset
    #   - schema: options = (source|seacrowd_[seacrowd_schema_name])
    #   - subset_id: subset id is the canonical name for the dataset (eg. smsa)
    #  where [seacrowd_schema_name] = (kb, pairs, qa, text, t2t)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indolem_sentiment_source",
            version=SOURCE_VERSION,
            description="indolem_sentiment source schema",
            schema="source",
            subset_id="indolem_sentiment",
        ),
        SEACrowdConfig(
            name="indolem_sentiment_seacrowd_text",
            version=SEACROWD_VERSION,
            description="indolem_sentiment Nusantara schema",
            schema="seacrowd_text",
            subset_id="indolem_sentiment",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indolem_sentiment_source"

    def _info(self) -> datasets.DatasetInfo:

        # Create the source schema; this schema will keep all keys/information/labels as close to the original dataset as possible.
        # You can arbitrarily nest lists and dictionaries.
        # For iterables, use lists over tuples or `datasets.Sequence`

        if self.config.schema == "source":
            features = datasets.Features({"sentence":datasets.Value("string"), "sentiment": datasets.Value("int32")})
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If you need to access the "source" or "seacrowd" config choice, that will be in self.config.name
        # LOCAL DATASETS: You do not need the dl_manager; you can ignore this argument. Make sure `gen_kwargs` in the return gets passed the right filepath
        # PUBLIC DATASETS: Assign your data-dir based on the dl_manager.
        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs; many examples use the download_and_extract method; see the DownloadManager docs here: https://huggingface.co/docs/datasets/package_reference/builder_classes.html#datasets.DownloadManager
        # dl_manager can accept any type of nested list/dict and will give back the same structure with the url replaced with the path to local files.

        # TODO: KEEP if your dataset is PUBLIC; remove if not
        urls = _URLS[_DATASETNAME]
        train_data = Path(dl_manager.download(urls['train']))
        test_data = Path(dl_manager.download(urls['test']))
        dev_data = Path(dl_manager.download(urls['dev']))
        
        # Not all datasets have predefined canonical train/val/test splits.
        # If your dataset has no predefined splits, use datasets.Split.TRAIN for all of the data.

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": train_data,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_data,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": dev_data,
                    "split": "dev",
                },
            ),
        ]

    # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`
    # TODO: change the args of this function to match the keys in `gen_kwargs`. You may add any necessary kwargs.

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # TODO: This method handles input defined in _split_generators to yield (key, example) tuples from the dataset.
        # The `key` is for legacy reasons (tfds) and is not important in itself, but must be unique for each example.
        # NOTE: For local datasets you will have access to self.config.data_dir and self.config.data_files

        with filepath.open('r', encoding='utf-8') as f:
            line = f.readline()
            id = 0
            while line:
                line = f.readline().strip()
                if len(line) == 0: break
                
                ex = {}
                id += 1
                sentence = line[:-2].strip('"')
                sentiment = int(line[-1])
                if self.config.schema == 'source':
                    ex = {'sentence': sentence, 'sentiment': sentiment}
                elif self.config.schema == 'seacrowd_text':
                    ex = {'id': str(id), 'text': str(sentence), 'label': self.label_classes[sentiment]}
                else:
                    raise ValueError(f"Invalid config: {self.config.name}")
                
                yield id, ex

# This template is based on the following template from the datasets package:
# https://github.com/huggingface/datasets/blob/master/templates/new_dataset_script.py


from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{hidayatullah2023corpus,
  title={Corpus creation and language identification for code-mixed Indonesian-Javanese-English Tweets},
  author={Hidayatullah, Ahmad Fathan and Apong, Rosyzie Anna and Lai, Daphne TC and Qazi, Atika},
  journal={PeerJ Computer Science},
  volume={9},
  pages={e1312},
  year={2023},
  publisher={PeerJ Inc.}
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "jav", "eng"]
_DATASETNAME = "ijelid"
_DESCRIPTION = """\
This is a code-mixed Indonesian-Javanese-English dataset for token-level
language identification. We named this dataset as IJELID
(Indonesian-Javanese-English Language Identification). This dataset contains
tweets that have been tokenized with the corresponding token and its language
label. There are seven language labels in the dataset, namely: ID (Indonesian)JV
(Javanese), EN (English), MIX_ID_EN (mixed Indonesian-English), MIX_ID_JV (mixed
Indonesian-Javanese), MIX_JV_EN (mixed Javanese-English), OTH (Other).
"""

_HOMEPAGE = "https://github.com/fathanick/Code-mixed-Indonesian-Javanese-English-Twitter-Data"
_LICENSE = Licenses.CC_BY_NC_SA_4_0.value
_URLS = {
    "train": "https://raw.githubusercontent.com/fathanick/Code-mixed-Indonesian-Javanese-English-Twitter-Data/main/train.tsv",
    "dev": "https://raw.githubusercontent.com/fathanick/Code-mixed-Indonesian-Javanese-English-Twitter-Data/main/val.tsv",
    "test": "https://raw.githubusercontent.com/fathanick/Code-mixed-Indonesian-Javanese-English-Twitter-Data/main/test.tsv",
}

_SUPPORTED_TASKS = [Tasks.TOKEN_LEVEL_LANGUAGE_IDENTIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IJELIDDataset(datasets.GeneratorBasedBuilder):
    """IJELID dataset from https://github.com/fathanick/Code-mixed-Indonesian-Javanese-English-Twitter-Data"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "seq_label"
    LABEL_CLASSES = ["ID", "JV", "EN", "MIX_ID_EN", "MIX_ID_JV", "MIX_JV_EN", "OTH"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        # No specific schema for the source, so for consistency,
        # I will use the same schema with SEACrowd
        features = schemas.seq_label_features(self.LABEL_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_files = {
            "train": Path(dl_manager.download_and_extract(_URLS["train"])),
            "dev": Path(dl_manager.download_and_extract(_URLS["dev"])),
            "test": Path(dl_manager.download_and_extract(_URLS["test"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"], "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["dev"], "split": "dev"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"], "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        with open(filepath, encoding="utf-8") as f:
            guid = 0
            tokens = []
            labels = []
            for line in f:
                if line == "" or line == "\n":
                    if tokens:
                        yield guid, {
                            "id": str(guid),
                            "tokens": tokens,
                            "labels": labels,
                        }
                        guid += 1
                        tokens = []
                        labels = []
                else:
                    # IJELID TSV are separated by \t
                    token, label = line.split("\t")
                    tokens.append(token)
                    labels.append(label.rstrip())

            # Last example
            if tokens:
                yield guid, {
                    "id": str(guid),
                    "tokens": tokens,
                    "labels": labels,
                }



from .image_text import features as image_text_features
from .kb import features as kb_features
from .tree import features as tree_features
from .pairs import features as pairs_features
from .pairs import features_with_continuous_label as pairs_features_score
from .pairs_multilabel import features as pairs_multi_features
from .qa import features as qa_features
from .imqa import features as imqa_features
from .self_supervised_pretraining import features as ssp_features
from .seq_label import features as seq_label_features
from .speech import features as speech_features
from .speech_multilabel import features as speech_multi_features
from .speech_text import features as speech_text_features
from .speech_to_speech import features as speech2speech_features
from .text import features as text_features
from .text_multilabel import features as text_multi_features
from .text_to_text import features as text2text_features
from .video import features as video_features
from .tod import features as tod_features

__all__ = [
    "image_text_features",
    "kb_features",
    "tree_features",
    "pairs_features",
    "pairs_features_score",
    "pairs_multi_features",
    "qa_features",
    "imqa_features",
    "ssp_features",
    "seq_label_features",
    "speech_features",
    "speech_multi_features",
    "speech_text_features",
    "speech2speech_features",
    "text_features",
    "text_multi_features",
    "text2text_features",
    "video_features",
    "tod_features",
]



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" IndQNER Dataset """

from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@misc{,
author = {Ria Hari Gusmita, Asep Fajar Firmansyah, Khodijah Khuliyah},
title = {{IndQNER: a NER Benchmark Dataset on Indonesian Translation of Quran}},
url = {https://github.com/dice-group/IndQNER},
year = {2022}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]
_DATASETNAME = "IndQNER"
_DESCRIPTION = """\
IndQNER is a NER dataset created by manually annotating the Indonesian translation of Quran text.
The dataset contains 18 named entity categories as follow:
    "Allah": Allah (including synonim of Allah such as Yang maha mengetahui lagi mahabijaksana)
    "Throne": Throne of Allah (such as 'Arasy)
    "Artifact": Artifact (such as Ka'bah, Baitullah)
    "AstronomicalBody": Astronomical body (such as bumi, matahari)
    "Event": Event (such as hari akhir, kiamat)
    "HolyBook": Holy book (such as AlQur'an)
    "Language": Language (such as bahasa Arab
    "Angel": Angel (such as Jibril, Mikail)
    "Person": Person (such as Bani Israil, Fir'aun)
    "Messenger": Messenger (such as Isa, Muhammad, Musa)
    "Prophet": Prophet (such as Adam, Sulaiman)
    "AfterlifeLocation": Afterlife location (such as Jahanam, Jahim, Padang Mahsyar)
    "GeographicalLocation": Geographical location (such as Sinai, negeru Babilonia)
    "Color": Color (such as kuning tua)
    "Religion": Religion (such as Islam, Yahudi, Nasrani)
    "Food": Food (such as manna, salwa)
"""

_HOMEPAGE = "https://github.com/dice-group/IndQNER"
_LICENSE = "Unknown"
_URLs = {
    "train": "https://raw.githubusercontent.com/dice-group/IndQNER/master/datasets/train.txt",
    "validation": "https://raw.githubusercontent.com/dice-group/IndQNER/master/datasets/dev.txt",
    "test": "https://raw.githubusercontent.com/dice-group/IndQNER/master/datasets/test.txt",
}
_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndqnerDataset(datasets.GeneratorBasedBuilder):
    """IndQNER is an Named Entity Recognition benchmark dataset on a niche domain i.e. Indonesian Translation of Quran."""

    label_classes = [
        "B-Allah",
        "B-Throne",
        "B-Artifact",
        "B-AstronomicalBody",
        "B-Event",
        "B-HolyBook",
        "B-Language",
        "B-Angel",
        "B-Person",
        "B-Messenger",
        "B-Prophet",
        "B-AfterlifeLocation",
        "B-GeographicalLocation",
        "B-Color",
        "B-Religion",
        "B-Food",
        "I-Allah",
        "I-Throne",
        "I-Artifact",
        "I-AstronomicalBody",
        "I-Event",
        "I-HolyBook",
        "I-Language",
        "I-Angel",
        "I-Person",
        "I-Messenger",
        "I-Prophet",
        "I-AfterlifeLocation",
        "I-GeographicalLocation",
        "I-Color",
        "I-Religion",
        "I-Food",
        "O",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indqner_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="NER dataset from Indonesian translation Quran source schema",
            schema="source",
            subset_id="indqner",
        ),
        SEACrowdConfig(
            name="indqner_seacrowd_seq_label",
            version=datasets.Version(_SOURCE_VERSION),
            description="NER dataset from Indonesian translation Quran Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id="indqner",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indqner_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("string"), "tokens": [datasets.Value("string")], "ner_tag": [datasets.Value("string")]})
        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_tsv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_tsv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_tsv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_tsv_path,
            "validation": validation_tsv_path,
            "test": test_tsv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        conll_dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for index, row in enumerate(conll_dataset):
                ex = {"index": str(index), "tokens": row["sentence"], "ner_tag": row["label"]}
                yield index, ex
        elif self.config.schema == "seacrowd_seq_label":
            for index, row in enumerate(conll_dataset):
                ex = {"id": str(index), "tokens": row["sentence"], "labels": row["label"]}
                yield index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from typing import Dict, List, Tuple

import datasets
import pandas

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@InProceedings{10.1007/978-3-031-21743-2_48,
author="Van Dinh, Co
and Luu, Son T.
and Nguyen, Anh Gia-Tuan",
editor="Nguyen, Ngoc Thanh
and Tran, Tien Khoa
and Tukayev, Ualsher
and Hong, Tzung-Pei
and Trawi{\'{n}}ski, Bogdan
and Szczerbicki, Edward",
title="Detecting Spam Reviews on Vietnamese E-Commerce Websites",
booktitle="Intelligent Information and Database Systems",
year="2022",
publisher="Springer International Publishing",
address="Cham",
pages="595--607",
abstract="The reviews of customers play an essential role in online shopping.
People often refer to reviews or comments of previous customers to decide whether
to buy a new product. Catching up with this behavior, some people create untruths and
illegitimate reviews to hoax customers about the fake quality of products. These are called
spam reviews, confusing consumers on online shopping platforms and negatively affecting online
shopping behaviors. We propose the dataset called ViSpamReviews, which has a strict annotation
procedure for detecting spam reviews on e-commerce platforms. Our dataset consists of two tasks:
the binary classification task for detecting whether a review is spam or not and the multi-class
classification task for identifying the type of spam. The PhoBERT obtained the highest results on
both tasks, 86.89%, and 72.17%, respectively, by macro average F1 score.",
isbn="978-3-031-21743-2"
}
"""

_LOCAL = False
_LANGUAGES = ["vie"]
_DATASETNAME = "vispamreviews"
_DESCRIPTION = """
The dataset was collected from leading online shopping platforms in Vietnam. Some of the most recent
selling products for each product category were selected and up to 15 reviews per product were collected.
Each review was then labeled as either NO-SPAM, SPAM-1 (fake review), SPAM-2 (review on brand only), or
SPAM-3 (irrelevant content).
"""

_HOMEPAGE = "https://github.com/sonlam1102/vispamdetection/"
_LICENSE = Licenses.CC_BY_NC_4_0.value
_URL = "https://raw.githubusercontent.com/sonlam1102/vispamdetection/main/dataset/vispamdetection_dataset.zip"

_Split_Path = {
    "train": "dataset/train.csv",
    "validation": "dataset/dev.csv",
    "test": "dataset/test.csv",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]  # Text Classification
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ViSpamReviewsDataset(datasets.GeneratorBasedBuilder):
    """
    The SeaCrowd dataloader for the review dataset shopping platforms in Vietnam (ViSpamReviews).
    """

    CLASS_LABELS = [0, 1]
    SPAM_TYPE_LABELS = [0, 1, 2, 3]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_spam_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema ",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_spam_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema ",
            schema="seacrowd_text",
            subset_id=f"{_DATASETNAME}_spam",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.name.endswith("source"):
            features = (datasets.Features
                (
                {"id": datasets.Value("int32"),
                 "text": datasets.Value("string"),
                 "label": datasets.Value("string"),
                 "spam_label": datasets.Value("string"),
                 "rating": datasets.Value("int32")
                 }
            ))

        elif self.config.name == "vispamreviews_seacrowd_text":
            features = schemas.text_features(label_names=self.CLASS_LABELS)
        elif self.config.name == "vispamreviews_spam_seacrowd_text":
            features = schemas.text_features(label_names=self.SPAM_TYPE_LABELS)
        else:
            raise ValueError(f"Invalid schema {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        file_paths = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["train"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["validation"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["test"])},
            ),
        ]

    def _generate_examples(self, filepath) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data_lines = pandas.read_csv(filepath)
        for rid, row in enumerate(data_lines.itertuples()):
            if self.config.name.endswith("source"):
                example = {"id": str(rid), "text": row.Comment, "label": row.Label, "spam_label": row.SpamLabel,
                           "rating": row.Rating}
            elif self.config.name == "vispamreviews_seacrowd_text":
                example = {"id": str(rid), "text": row.Comment, "label": row.Label}
            elif self.config.name == "vispamreviews_spam_seacrowd_text":
                example = {"id": str(rid), "text": row.Comment, "label": row.SpamLabel}
            else:
                raise ValueError(f"Invalid schema {self.config.schema}")
            yield rid, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from itertools import chain
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)

_CITATION = """\
@inproceedings{sakti-tcast-2008,
    title = "Development of {I}ndonesian Large Vocabulary Continuous Speech Recognition System within {A-STAR} Project",
    author = "Sakti, Sakriani and Kelana, Eka and Riza, Hammam and Sakai, Shinsuke and Markov, Konstantin and Nakamura, Satoshi",
    booktitle = "Proc. IJCNLP Workshop on Technologies and Corpora for Asia-Pacific Speech Translation (TCAST)",
    year = "2008",
    pages = "19--24"
    address = "Hyderabad, India"
}

@inproceedings{sakti-icslp-2004,
    title = "Indonesian Speech Recognition for Hearing and Speaking Impaired People",
    author = "Sakti, Sakriani and Hutagaol, Paulus and Arman, Arry Akhmad and Nakamura, Satoshi",
    booktitle = "Proc. International Conference on Spoken Language Processing (INTERSPEECH - ICSLP)",
    year = "2004",
    pages = "1037--1040"
    address = "Jeju Island, Korea"
}

@article{sakti-s2st-csl-2013,
    title = "{A-STAR}: Toward Translating Asian Spoken Languages",
    author = "Sakti, Sakriani and Paul, Michael and Finch, Andrew and Sakai, Shinsuke and Thang, Tat Vu, and Kimura, Noriyuki and Hori, Chiori and Sumita, Eiichiro and Nakamura, Satoshi and Park, Jun and Wutiwiwatchai, Chai and Xu, Bo and Riza, Hammam and Arora, Karunesh and Luong, Chi Mai and Li, Haizhou",
    journal = "Special issue on Speech-to-Speech Translation, Computer Speech and Language Journal",
    volume = "27",
    number ="2",
    pages = "509--527",
    year = "2013",
    publisher = "Elsevier"
}
"""

_DATASETNAME = "indspeech_news_lvcsr"
_LANGUAGES = ["ind"] 
_LOCAL = False

_DESCRIPTION = """\
This is the first Indonesian speech dataset for large vocabulary continuous speech recognition (LVCSR) with more than 40 hours of speech and 400 speakers [Sakti et al., 2008]. R&D Division of PT Telekomunikasi Indonesia (TELKOMRisTI) developed the data in 2005-2006, in collaboration with Advanced Telecommunication Research Institute International (ATR) Japan, as the continuation of the Asia-Pacific Telecommunity (APT) project [Sakti et al., 2004]. It has also been successfully used for developing Indonesian LVCSR in the Asian speech translation advanced research (A-STAR) project [Sakti et al., 2013].
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_news_lvcsr"

_LICENSE = "CC BY-NC-SA 4.0"

URL_TEMPLATE = {
    "lst": "https://raw.githubusercontent.com/s-sakti/data_indsp_news_lvcsr/main/lst/",  # transcript.lst
    "speech": "https://github.com/s-sakti/data_indsp_news_lvcsr/raw/main/speech/",  # Ind3/Ind304.zip~Ind400.zip
    "text": "https://github.com/s-sakti/data_indsp_news_lvcsr/raw/main/text/",  # all_transcript.zip
}

_URLS = {
    "lst_spk_Ind": [URL_TEMPLATE["lst"] + "spk_Ind" + str(n) + ".lst" for n in range(0, 4)],
    "lst_spk_all": URL_TEMPLATE["lst"] + "spk_all.lst",
    "lst_spk_test": URL_TEMPLATE["lst"] + "spk_test.lst",
    "lst_spk_train": URL_TEMPLATE["lst"] + "spk_train.lst",
    "lst_transcript": URL_TEMPLATE["lst"] + "transcript.lst",
    "speech_Ind": [URL_TEMPLATE["speech"] + "Ind" + str(n) + "/Ind" + str(p).zfill(3) + ".zip" for n in range(0, 4) for p in range(n * 100 + 1, n * 100 + 101)],
    "transcript_all": URL_TEMPLATE["text"] + "all_transcript.zip",
    "transcript_spk": URL_TEMPLATE["text"] + "spk_transcript.zip",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndSpeechNewsLVCSR(datasets.GeneratorBasedBuilder):
    """Indonesian automatic speech recognition with several local accents reading short news sentences"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indspeech_news_lvcsr_source",
            version=SOURCE_VERSION,
            description="indspeech_news_lvcsr source schema",
            schema="source",
            subset_id="indspeech_news_lvcsr",
        ),
        SEACrowdConfig(
            name="indspeech_news_lvcsr_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description="indspeech_news_lvcsr Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="indspeech_news_lvcsr",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indspeech_news_lvcsr_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "spk_id": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                    "accent": datasets.Value("string"),
                    "type": datasets.Value("string"),
                    "txt_id": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        lst_spk_train = _URLS["lst_spk_train"]
        lst_spk_test = _URLS["lst_spk_test"]
        transcript = _URLS["lst_transcript"]
        audio_urls = _URLS["speech_Ind"]

        lst_spk_train_dir = Path(dl_manager.download_and_extract(lst_spk_train))
        lst_spk_test_dir = Path(dl_manager.download_and_extract(lst_spk_test))
        transcript_dir = Path(dl_manager.download_and_extract(transcript))
        audio_files_dir = [Path(dl_manager.download_and_extract(aud_url)) / aud_url.split("/")[-1][:-4] for aud_url in audio_urls]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {"lst_spk": lst_spk_train_dir, "transcript": transcript_dir, "aud_files": audio_files_dir},
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {"lst_spk": lst_spk_test_dir, "transcript": transcript_dir, "aud_files": audio_files_dir},
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        speaker_ids = open(filepath["lst_spk"], "r").readlines()
        speaker_ids = [id.replace("\n", "") for id in speaker_ids]
        speech_folders = [aud_folder for aud_folder in filepath["aud_files"] if aud_folder.name in speaker_ids]
        speech_files = list(chain(*[list(map((str(speech_folder) + "/").__add__, os.listdir(speech_folder))) for speech_folder in speech_folders]))

        print(speech_files[0])

        transcript = open(filepath["transcript"], "r").readlines()
        transcript = [sentence.replace("\n", "") for sentence in transcript]

        for key, aud_file in enumerate(speech_files):
            aud_id = str(Path(aud_file).name).split("\\")[-1][:-4]
            aud_info = aud_id.split("_")

            if self.config.schema == "source":
                row = {
                    "spk_id": aud_info[0],
                    "gender": aud_info[1],
                    "accent": aud_info[2],
                    "type": aud_info[3],
                    "txt_id": aud_info[5],
                    "audio": aud_file,
                    "text": transcript[int(aud_info[5])],
                }
                yield key, row

            elif self.config.schema == "seacrowd_sptext":
                row = {
                    "id": aud_id,
                    "path": aud_file,
                    "audio": aud_file,
                    "text": transcript[int(aud_info[5])],
                    "speaker_id": aud_info[0],
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": aud_info[1],
                    },
                }
                yield key, row
            else:
                raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



"""
Text Pairs Multilabel Schema
"""
import datasets


def features(label_names=["Yes", "No"]):
    return datasets.Features(
        {
            "id": datasets.Value("string"),
            "text_1": datasets.Value("string"),
            "text_2": datasets.Value("string"),
            "label": datasets.Sequence(datasets.ClassLabel(names=label_names)),
        }
    )


import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import json

_CITATION = """\
@inproceedings{koto2020liputan6,
  title={Liputan6: A Large-scale Indonesian Dataset for Text Summarization},
  author={Koto, Fajri and Lau, Jey Han and Baldwin, Timothy},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  pages={598--608},
  year={2020}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "liputan6"

_DESCRIPTION = """
A large-scale Indonesian summarization dataset consisting of harvested articles from Liputan6.com, an online news portal, resulting in 215,827 document-summary pairs.
"""

_HOMEPAGE = "https://github.com/fajri91/sum_liputan6"

_LICENSE = "CC-BY-SA 4.0"

_URLS = {
    _DATASETNAME: "https://storage.googleapis.com/babert-pretraining/IndoNLG_finals/downstream_task/downstream_task_datasets.zip",
}

_SUPPORTED_TASKS = [Tasks.SUMMARIZATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class Liputan6(datasets.GeneratorBasedBuilder):
    """A large-scale Indonesian summarization dataset consisting of harvested articles from Liputan6.com, an online news portal, resulting in 215,827 document-summary pairs."""


    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    
    TYPE_LIST = ['canonical', 'xtreme']
    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name="liputan6_{fold_name}_source".format(fold_name=i),
                version=_SOURCE_VERSION,
                description="liputan6 source schema",
                schema="source",
                subset_id="liputan6_{fold_name}".format(fold_name=i),
            ) for i in TYPE_LIST
        ]
        +
        [
            SEACrowdConfig(
            name="liputan6_{fold_name}_seacrowd_t2t".format(fold_name=i),
            version=_SEACROWD_VERSION,
            description="liputan6 Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="liputan6_{fold_name}".format(fold_name=i),
        ) for i in TYPE_LIST
        ]
    )
    DEFAULT_CONFIG_NAME = "liputan6_canonical_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
               {
                   "document": datasets.Value("string"),
                   "id": datasets.Value("string"),
                   "summary": datasets.Value("string")
               }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _get_fold_name(self):
        subset_id = self.config.subset_id
        idx_fold = subset_id.index("_")
        file_id = subset_id[(idx_fold + 1):]
        return file_id

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        fold_name = self._get_fold_name()

        urls = _URLS[_DATASETNAME]

        data_dir = Path(dl_manager.download_and_extract(urls))

        location = {
            "train": "IndoNLG_downstream_tasks/liputan6/{fold_name}_train.json",
            "test": "IndoNLG_downstream_tasks/liputan6/{fold_name}_test.json",
            "dev": "IndoNLG_downstream_tasks/liputan6/{fold_name}_dev.json"
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["train"].format(fold_name=fold_name)),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["test"].format(fold_name=fold_name)),
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, location["dev"].format(fold_name=fold_name)),
                    "split": "dev",
                },
            ),
        ]
    

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:

        if self.config.schema == "source":
            
            if "xtreme_train.json" in filepath:
                with open(filepath) as f:
                    lines = f.read().split("{")
                    LEN = len(lines)
                    for i, line in enumerate(lines):
                        if 0 < i < LEN-1:
                            idx = line.index("}")
                            line = "{"+line[:idx+1]
                            each_data = json.loads(line)
                            ex = {
                                "id": each_data["id"],
                                "document": each_data['text'],
                                "summary": each_data['label']
                            }
                            yield each_data["id"], ex

            else:
                with open(filepath) as f:
                    data =  json.load(f)
                    for i, each_data in enumerate(data):
                        ex = {
                            "id": each_data["id"],
                            "document": each_data['text'],
                            "summary": each_data['label']
                        }
                        yield each_data["id"], ex

        elif self.config.schema == "seacrowd_t2t":
            if "xtreme_train.json" in filepath:
                with open(filepath) as f:
                    lines = f.read().split("{")
                    LEN = len(lines)
                    for i, line in enumerate(lines):
                        if 0 < i < LEN-1:
                            idx = line.index("}")
                            line = "{"+line[:idx+1]
                            each_data = json.loads(line)
                            ex = {
                                "id": each_data["id"],
                                "text_1": each_data['text'],
                                "text_2": each_data['label'],
                                "text_1_name": "document",
                                "text_2_name": "summary"
                            }
                            yield each_data["id"], ex

            else:
                with open(filepath) as f:
                    data =  json.load(f)
                    for i, each_data in enumerate(data):
                        ex = {
                            "id": each_data["id"],
                            "text_1": each_data['text'],
                            "text_2": each_data['label'],
                            "text_1_name": "document",
                            "text_2_name": "summary"
                        }
                        yield each_data["id"], ex
            
            
            



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@article{hidayatullah2020attention,
  title={Attention-based cnn-bilstm for dialect identification on javanese text},
  author={Hidayatullah, Ahmad Fathan and Cahyaningtyas, Siwi and Pamungkas, Rheza Daffa},
  journal={Kinetik: Game Technology, Information System, Computer Network, Computing, Electronics, and Control},
  pages={317--324},
  year={2020}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "jadi_ide"

_DESCRIPTION = """\
The JaDi-Ide dataset is a Twitter dataset for Javanese dialect identification, containing 16,498 
data samples. The dialect is classified into `Standard Javanese`, `Ngapak Javanese`, and `East 
Javanese` dialects.
"""

_HOMEPAGE = "https://github.com/fathanick/Javanese-Dialect-Identification-from-Twitter-Data"
_LICENSE = "Unknown"
_URLS = {
    _DATASETNAME: "https://github.com/fathanick/Javanese-Dialect-Identification-from-Twitter-Data/raw/main/Update 16K_Dataset.xlsx",
}
# TODO check supported tasks
_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class JaDi_Ide(datasets.GeneratorBasedBuilder):
    """The JaDi-Ide dataset is a Twitter dataset for Javanese dialect identification, containing 16,498 
    data samples."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="jadi_ide_source",
            version=SOURCE_VERSION,
            description="JaDi-Ide source schema",
            schema="source",
            subset_id="jadi_ide",
        ),
        SEACrowdConfig(
            name="jadi_ide_seacrowd_text",
            version=SEACROWD_VERSION,
            description="JaDi-Ide Nusantara schema",
            schema="seacrowd_text",
            subset_id="jadi_ide",
        ),
    ]

    DEFAULT_CONFIG_NAME = "jadi_ide_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"), 
                    "text": datasets.Value("string"), 
                    "label": datasets.Value("string")
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["Jawa Timur", "Jawa Standar", "Jawa Ngapak"])
   

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download(urls))
        data_files = {"train": base_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_excel(filepath)
        df.columns = ["id", "text", "label"]

        if self.config.schema == "source":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text": row.text,
                    "label": row.label,
                }
                yield idx, ex

        elif self.config.schema == "seacrowd_text":
            for idx, row in enumerate(df.itertuples()):
                ex = {
                    "id": str(idx),
                    "text": row.text,
                    "label": row.label,
                }
                yield idx, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



"""
This new update refers to the this HF dataloader script
https://huggingface.co/datasets/csebuetnlp/xlsum/blob/main/xlsum.py
while conforming to SEACrowd schema
"""

import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@inproceedings{valk2021slt,
  title={{VoxLingua107}: a Dataset for Spoken Language Recognition},
  author={J{\"o}rgen Valk and Tanel Alum{\"a}e},
  booktitle={Proc. IEEE SLT Workshop},
  year={2021},
}
"""

_LOCAL = False
_LANGUAGES = ["ceb", "ind", "jav", "khm", "lao", "zlm", "mya", "sun", "tha", "tgl", "vie", "war"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LANG_TO_DATASOURCE_LANG = {
    "ceb": "ceb",
    "ind": "id",
    "jav": "jw",
    "khm": "km",
    "lao": "lo",
    "zlm": "ms",
    "mya": "my",
    "sun": "su",
    "tha": "th",
    "tgl": "tl",
    "vie": "vi",
    "war": "war"}

_DATASETNAME = "voxlingua"

_DESCRIPTION = """\
VoxLingua107 is a comprehensive speech dataset designed for training spoken language identification models.
It comprises short speech segments sourced from YouTube videos, labeled based on the language indicated in the video
title and description. The dataset covers 107 languages and contains a total of 6628 hours of speech data,
averaging 62 hours per language. However, the actual amount of data per language varies significantly.
Additionally, there is a separate development set consisting of 1609 speech segments from 33 languages,
validated by at least two volunteers to ensure the accuracy of language representation.
"""

_HOMEPAGE = "https://bark.phon.ioc.ee/voxlingua107/"

_LICENSE = Licenses.CC_BY_4_0.value

_URLS = "https://bark.phon.ioc.ee/voxlingua107/{identifier}.zip"

_SUPPORTED_TASKS = [Tasks.SPEECH_LANGUAGE_IDENTIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def construct_configs() -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects, and returns the list.

    output:
        a list of `SEACrowdConfig` objects.
    """

    # set output var
    config_list = []

    # construct zipped arg for config instantiation
    CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]
    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema",
            schema=f"{config_name_prefix}",
            subset_id=f"{config_name_prefix}",
        )
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for task_obj, config_name_suffix in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=f"{config_name_prefix}_{config_name_suffix}",
            )
        ]
    return config_list


class VoxLinguaDataset(datasets.GeneratorBasedBuilder):
    """Speech Lang ID on dataset VoxLingua."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = construct_configs()

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            # since the source only contains audio folder structure,
            # we will define it using simplified ver of SEACrowd speech_features schema
            features = datasets.Features({
                "id": datasets.Value("string"),
                "path": datasets.Value("string"),
                "audio": datasets.Audio(sampling_rate=16_000),
                "labels": datasets.ClassLabel(names=_LANGUAGES)})

        elif self.config.schema == "seacrowd_speech":
            features = schemas.speech_features(label_names=_LANGUAGES)

        else:
            raise ValueError(f"Unexpected self.config.schema of {self.config.schema}!")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # since this is a Speech LID, all languages must be downloaded in a single lists
        # for train data, the identifier is a lang_code defined in `_LANG_TO_DATASOURCE_LANG`
        train_url_list = [_URLS.format(identifier=_LANG_TO_DATASOURCE_LANG[lang_val]) for lang_val in _LANGUAGES]
        train_data_dir = dl_manager.download_and_extract(train_url_list)

        # for val data, the `dev.zip` doesn't contain any data indicated in _LANGUAGES

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_data_dir,
                },
            )
        ]

    def _generate_examples(self, filepath: List[Path]) -> Tuple[int, Dict]:

        # this is defined as -1 so that in the first loop it will have value of 0
        example_idx = -1

        for idx, child_path in enumerate(filepath):
            # check for 2 things:

            # 1. for every filepath list element (which contain 1 lang data), it will contain only 1 subdir and named its lang code in source
            first_level_rel_dir = os.listdir(child_path)
            expected_lang_label = _LANG_TO_DATASOURCE_LANG[_LANGUAGES[idx]]
            assert first_level_rel_dir == [expected_lang_label], f"The structure of path is unexpected! Expected {[expected_lang_label]} got: {first_level_rel_dir}"

            # 2. within the first_level_dir, all of them are file (no directory)
            first_level_dir = os.path.join(child_path, first_level_rel_dir[0])
            second_level_dir = os.listdir(first_level_dir)
            assert not all(os.path.isdir(expected_file) for expected_file in second_level_dir), f"Found directory within folder {first_level_dir}!"

            # extract sound data with format ".wav"
            wav_files = [os.path.join(first_level_dir, file) for file in second_level_dir if file.endswith(".wav")]

            if self.config.schema == "source":
                for _fp in wav_files:
                    example_idx += 1
                    ex = {"id": example_idx, "path": _fp, "audio": _fp, "labels": _LANGUAGES.index(expected_lang_label)}
                    yield example_idx, ex

            elif self.config.schema == "seacrowd_speech":
                for _fp in wav_files:
                    example_idx += 1
                    # audio = {"path": file, "bytes": file.read()}
                    ex = {
                        "id": example_idx,
                        "path": _fp,
                        "audio": _fp,
                        "speaker_id": "",
                        "labels": _LANGUAGES.index(expected_lang_label),
                        "metadata": {
                            # unavailable, filled with default val
                            "speaker_age": -1,
                            "speaker_gender": "",
                        },
                    }
                    yield example_idx, ex

            else:
                raise ValueError(f"Invalid config schema of {self.config.schema}!")



from pathlib import Path
from sys import stdout
from tempfile import tempdir
from typing import List
from unicodedata import category

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "wrete"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@INPROCEEDINGS{8904199,
    author={Purwarianti, Ayu and Crisdayanti, Ida Ayu Putu Ari},
    booktitle={2019 International Conference of Advanced Informatics: Concepts, Theory and Applications (ICAICTA)},
    title={Improving Bi-LSTM Performance for Indonesian Sentiment Analysis Using Paragraph Vector},
    year={2019},
    pages={1-5},
    doi={10.1109/ICAICTA.2019.8904199}
}

@inproceedings{wilie2020indonlu,
  title={IndoNLU: Benchmark and Resources for Evaluating Indonesian Natural Language Understanding},
  author={Wilie, Bryan and Vincentio, Karissa and Winata, Genta Indra and Cahyawijaya, Samuel and Li, Xiaohong and Lim, Zhi Yuan and Soleman, Sidik and Mahendra, Rahmad and Fung, Pascale and Bahar, Syafri and others},
  booktitle={Proceedings of the 1st Conference of the Asia-Pacific Chapter of the Association for Computational Linguistics and the 10th International Joint Conference on Natural Language Processing},
  pages={843--857},
  year={2020}
}
"""


_DESCRIPTION = """\
WReTe, The Wiki Revision Edits Textual Entailment dataset (Setya and Mahendra, 2018) consists of 450 sentence pairs constructed from Wikipedia revision history. The dataset contains pairs of sentences and binary semantic relations between the pairs. The data are labeled as entailed when the meaning of the second sentence can be derived from the first one, and not entailed otherwise
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "Creative Common Attribution Share-Alike 4.0 International"

_URLs = {
    "train": "https://github.com/IndoNLP/indonlu/raw/master/dataset/wrete_entailment-ui/train_preprocess.csv",
    "validation": "https://github.com/IndoNLP/indonlu/raw/master/dataset/wrete_entailment-ui/valid_preprocess.csv",
    "test": "https://github.com/IndoNLP/indonlu/raw/master/dataset/wrete_entailment-ui/test_preprocess_masked_label.csv",
}

_SUPPORTED_TASKS = [Tasks.TEXTUAL_ENTAILMENT]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class WReTe(datasets.GeneratorBasedBuilder):
    """WReTe consists of premise, hypothesis, category, and label. The Data are labeled as entailed when the meaning of the second sentence can be derived from the first one, and not entailed otherwise"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="wrete_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="WReTe source schema",
            schema="source",
            subset_id="wrete",
        ),
        SEACrowdConfig(
            name="wrete_seacrowd_pairs",
            version=datasets.Version(_SEACROWD_VERSION),
            description="WReTe Nusantara schema",
            schema="seacrowd_pairs",
            subset_id="wrete",
        ),
    ]

    DEFAULT_CONFIG_NAME = "wrete_source"
    labels = ["NotEntail", "Entail_or_Paraphrase"]

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features({"index": datasets.Value("int32"), 
                                        "sent_A": datasets.Value("string"), 
                                        "sent_B": datasets.Value("string"),
                                        "category" : datasets.Value("string"),
                                        "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_pairs":
            features = schemas.pairs_features(self.labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_csv_path = Path(dl_manager.download_and_extract(_URLs["train"]))
        validation_csv_path = Path(dl_manager.download_and_extract(_URLs["validation"]))
        test_csv_path = Path(dl_manager.download_and_extract(_URLs["test"]))
        data_files = {
            "train": train_csv_path,
            "validation": validation_csv_path,
            "test": test_csv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": data_files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_files["test"]},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        df = pd.read_csv(filepath, sep=",", quotechar='"').reset_index()

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {"index": row.index,
                     "sent_A" : row.sent_A,
                     "sent_B" : row.sent_B,
                     "category" : row.category, 
                     "label" : row.label
                }
                yield row.index, ex
        elif self.config.schema == "seacrowd_pairs":
            for row in df.itertuples():
                ex = {
                    "id": row.index,
                    "text_1": row.sent_A,
                    "text_2": row.sent_B,
                    "label": row.label
                }
                yield row.index, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses

_CITATION = """\
@INPROCEEDINGS{ramli2022indokepler,
    author={Ramli, Inigo and Krisnadhi, Adila Alfa and Prasojo, Radityo Eko},
    booktitle={2022 7th International Workshop on Big Data and Information Security (IWBIS)},
    title={IndoKEPLER, IndoWiki, and IndoLAMA: A Knowledge-enhanced Language Model, Dataset, and Benchmark for the Indonesian Language},
    year={2022},
    volume={},
    number={},
    pages={19-26},
    doi={10.1109/IWBIS56557.2022.9924844}}
"""

_DATASETNAME = "indowiki"
_DESCRIPTION = """\
IndoWiki is a knowledge-graph dataset taken from WikiData and aligned with Wikipedia Bahasa Indonesia as it's corpus.
"""
_HOMEPAGE = "https://github.com/IgoRamli/IndoWiki"
_LANGUAGES = ["ind"]
_LICENSE = Licenses.MIT.value
_LOCAL = False

_URLS = {
    "inductive": {
        "train": "https://drive.google.com/uc?export=download&id=1S3vNx9By5CWKGkObjtXaI6Jr4xri2Tz3",
        "valid": "https://drive.google.com/uc?export=download&id=1cP-zDIxp9a-Bw9uYd40K9IN-4wg4dOgy",
        "test": "https://drive.google.com/uc?export=download&id=1pLcoJgYmgQiN4Gv9tRcI26zM7-OgHcuZ",
    },
    "transductive": {
        "train": "https://drive.google.com/uc?export=download&id=1KXDVwboo1h2yk_kAqv7IPYnHXCK6g-6X",
        "valid": "https://drive.google.com/uc?export=download&id=1eRwpuRPYOnA-7FZ-YNZjRJ2DHuJsfUIE",
        "test": "https://drive.google.com/uc?export=download&id=1cy9FwDMB_U-js8P8u4IWolvNeIFkQVDh",
    },
    "text": "https://drive.usercontent.google.com/download?id=1YC4P_IPSo1AsEwm5Z_4GBjDdwCbvokxX&export=download&authuser=0&confirm=t&uuid=36aa95f5-e1b6-43c1-a34f-754d14d8b473&at=APZUnTWD7fwarBs4ZVRy_QdKbDXi%3A1709478240158",
}

# none of the tasks in schema
# dataset is used to learn knowledge embedding
_SUPPORTED_TASKS = []

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndoWiki(datasets.GeneratorBasedBuilder):
    """IndoWiki knowledge base dataset from https://github.com/IgoRamli/IndoWiki"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        # inductive setting
        SEACrowdConfig(
            name=f"{_DATASETNAME}_inductive_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        # transductive setting
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "ent1": datasets.Value("string"),
                    "ent2": datasets.Value("string"),
                    "ent1_text": datasets.Value("string"),
                    "ent2_text": datasets.Value("string"),
                    "relation": datasets.Value("string"),
                }
            )

        else:
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        if "inductive" in self.config.name:
            setting = "inductive"
            data_paths = {
                "inductive": {
                    "train": Path(dl_manager.download_and_extract(_URLS["inductive"]["train"])),
                    "valid": Path(dl_manager.download_and_extract(_URLS["inductive"]["valid"])),
                    "test": Path(dl_manager.download_and_extract(_URLS["inductive"]["test"])),
                },
                "text": Path(dl_manager.download_and_extract(_URLS["text"])),
            }
        else:
            setting = "transductive"
            data_paths = {
                "transductive": {
                    "train": Path(dl_manager.download_and_extract(_URLS["transductive"]["train"])),
                    "valid": Path(dl_manager.download_and_extract(_URLS["transductive"]["valid"])),
                    "test": Path(dl_manager.download_and_extract(_URLS["transductive"]["test"])),
                },
                "text": Path(dl_manager.download_and_extract(_URLS["text"])),
            }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "triplets_filepath": data_paths[setting]["train"],
                    "text_filepath": data_paths["text"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "triplets_filepath": data_paths[setting]["test"],
                    "text_filepath": data_paths["text"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "triplets_filepath": data_paths[setting]["valid"],
                    "text_filepath": data_paths["text"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, triplets_filepath: Path, text_filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        # read triplets file
        with open(triplets_filepath, "r", encoding="utf-8") as triplets_file:
            triplets_data = triplets_file.readlines()
        triplets_data = [s.strip("\n").split("\t") for s in triplets_data]

        # read text description file
        with open(text_filepath, "r", encoding="utf-8") as text_file:
            text_data = text_file.readlines()
        # dictionary of entity: text description of entity
        text_dict = {s.split("\t")[0]: s.split("\t")[1].strip("\n") for s in text_data}

        num_sample = len(triplets_data)

        for i in range(num_sample):
            if self.config.schema == "source":
                example = {
                    "id": str(i),
                    "ent1": triplets_data[i][0],
                    "ent2": triplets_data[i][2],
                    "ent1_text": text_dict[triplets_data[i][0]],
                    "ent2_text": text_dict[triplets_data[i][2]],
                    "relation": triplets_data[i][1],
                }

            yield i, example



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import json

_CITATION = """\
@misc{quoraFirstQuora,
	author = {},
	title = {{F}irst {Q}uora {D}ataset {R}elease: {Q}uestion {P}airs --- quoradata.quora.com},
	howpublished = {https://quoradata.quora.com/First-Quora-Dataset-Release-Question-Pairs},
	year = 2017,
	note = {Online},
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "id_qqp"

_DESCRIPTION = """\
Quora Question Pairs (QQP) dataset consists of over 400,000 question pairs, 
and each question pair is annotated with a binary value indicating whether 
the two questions are paraphrase of each other. This dataset is translated 
version of QQP to Indonesian Language.
"""

_HOMEPAGE = "https://github.com/louisowen6/quora_paraphrasing_id"

_LICENSE = "Apache License, Version 2.0"

_URLS = {
    _DATASETNAME: [
        "https://github.com/louisowen6/quora_paraphrasing_id/raw/main/ID_Quora_Paraphrasing_train.json",
        "https://github.com/louisowen6/quora_paraphrasing_id/raw/main/ID_Quora_Paraphrasing_val.json",
    ]
}

_SUPPORTED_TASKS = [Tasks.PARAPHRASING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IdQuoraQuestionPairs(datasets.GeneratorBasedBuilder):
    """
    Quora Question Pairs (QQP) dataset consists of over 400,000 question pairs, 
    and each question pair is annotated with a binary value indicating whether 
    the two questions are paraphrase of each other. This dataset is translated 
    version of QQP to Indonesian Language.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    
    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_qqp_source",
            version=SOURCE_VERSION,
            description="ID QQP source schema",
            schema="source",
            subset_id="id_qqp",
        ),
        SEACrowdConfig(
            name="id_qqp_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="ID QQP Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="id_qqp",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_qqp_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
               {
                   "id": datasets.Value("string"),
                   "question_1": datasets.Value("string"),
                   "question_2": datasets.Value("string")
               }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )


    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir[0],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir[1],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        
        with open(filepath, "r") as f:
            lines = f.readlines()

        if self.config.schema == "source":
            
            for i, line in enumerate(lines):
                line = json.loads(line.strip())
                
                sample = {
                    "id": str(i),
                    "question_1": line["question_1"],
                    "question_2": line["question_2"]
                }
                yield i, sample

        elif self.config.schema == "seacrowd_t2t":
            
            for i, line in enumerate(lines):
                line = json.loads(line.strip())
                
                sample = {
                    "id": str(i),
                    "text_1": line["question_1"],
                    "text_2": line["question_2"],
                    "text_1_name": "question_1",
                    "text_2_name": "question_2"
                }
                yield i, sample



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{inproceedings,
author = {Koto, Fajri and Rahmaningtyas, Gemala},
year = {2017},
month = {12},
pages = {},
title = {InSet Lexicon: Evaluation of a Word List for Indonesian Sentiment Analysis in Microblogs},
doi = {10.1109/IALP.2017.8300625}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "inset_lexicon"

_DESCRIPTION = """\
InSet, an Indonesian sentiment lexicon built to identify written opinion and categorize it into positive or negative opinion,
which could be utilized to analyze public sentiment towards particular topic, event, or product. Composed using collection
of words from Indonesian tweet, InSet was constructed by manually weighting each words and enhanced by adding stemming and synonym set
"""

_HOMEPAGE = "https://www.researchgate.net/publication/321757985_InSet_Lexicon_Evaluation_of_a_Word_List_for_Indonesian_Sentiment_Analysis_in_Microblogs"
_LICENSE = "Unknown"
_URLS = {_DATASETNAME: "https://github.com/fajri91/InSet/archive/refs/heads/master.zip"}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class InsetLexicon(datasets.GeneratorBasedBuilder):
    """InSet, an Indonesian sentiment lexicon built to identify written opinion and categorize it into positive or negative opinion"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="inset_lexicon_source",
            version=SOURCE_VERSION,
            description="Inset Lexicon source schema",
            schema="source",
            subset_id="inset_lexicon",
        ),
        SEACrowdConfig(
            name="inset_lexicon_seacrowd_text",
            version=SEACROWD_VERSION,
            description="Inset Lexicon Nusantara schema",
            schema="seacrowd_text",
            subset_id="inset_lexicon",
        ),
    ]

    DEFAULT_CONFIG_NAME = "inset_lexicon_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"word": datasets.Value("string"), "weight": datasets.Value("string")})
        elif self.config.schema == "seacrowd_text":
            labels = list(range(-5, 6, 1))
            labels = [str(label) for label in labels]
            features = schemas.text_features(labels)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download_and_extract(urls)) / "InSet-master"
        positive_df = pd.read_csv(base_dir / "positive.tsv", sep="\t")
        negative_df = pd.read_csv(base_dir / "negative.tsv", sep="\t")
        merged_df = pd.concat([positive_df, negative_df]).reset_index(drop=True)
        merged_data_dir = base_dir / "dataset.tsv"
        merged_df.to_csv(merged_data_dir, sep="\t")

        data_files = {"train": merged_data_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        df = pd.read_csv(filepath, sep="\t", encoding="ISO-8859-1")
        df.columns = ["id", "word", "weight"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "word": row.word,
                    "weight": str(int(row.weight)),
                }
                yield row.id, ex

        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.word,
                    "label": str(int(row.weight)),
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")
